#include <defines.h>

#include <avr/io.h>
#include <avr/interrupt.h>

#include <timer.h>






//---------- Funktionen ----------

// Initialisiert die Timerflags & Timer
void timer0_init(void) {
	T1ms_flag	= 0;
	T10ms_flag	= 0;
	T100ms_flag	= 0;
	
	timer0_set_value(0x00);
	timer0_set_ctcmode();
	timer0_set_prescaler(0x02);			// set prescaler to 8
	timer0_set_reload_value(75 - 1); 	// 1ms-Interrupt: 8 * 75 * 25 / 15MHz
	timer0_enable_OCM_interrupt();
}

// Aktiviert den Clear on Timer Compare Match Modus
void timer0_set_ctcmode(void) {
	SETBIT(TCCR0, WGM01);
	CLRBIT(TCCR0, WGM00);
}

// Setzt den Vorteiler fr Timer0
void timer0_set_prescaler(uint8_t psc_setting) {
	TCCR0 |= psc_setting;
}

// Hlt Timer0 an
void timer0_stop(void) {
	CLRBIT(TCCR0, CS00);
	CLRBIT(TCCR0, CS01);
	CLRBIT(TCCR0, CS02);
}

// Aktiviert Timer0 (mit ungeteiltem Taktsignal)
void timer0_start(void) {
	timer0_set_prescaler(0x01);
}

// Schreibt den Reload-Wert
void timer0_set_reload_value(uint8_t rld) {
	OCR0 = rld;
}

// Schreibt Wert in das Timerregister
void timer0_set_value(uint8_t val) {
	TCNT0 = val;
}

// Gibt den aktuellen Inhalt des Timerregisters zureuck
uint8_t timer0_get_value(void) {
	return TCNT0;
}

// Aktiviert den Output Compare Match Interrupt
void timer0_enable_OCM_interrupt(void) {
	SETBIT(TIMSK, OCIE0);
}

// Deaktiviert den Output Compare Match Interrupt
void timer0_disable_OCM_interrupt(void) {
	CLRBIT(TIMSK, OCIE0);
}





//---------- Interruptroutinen ----------

// Output Compare Match Interrupt
ISR(TIMER0_COMP_vect) {
	static uint8_t T1ms_counter		= 0;
	static uint8_t T10ms_counter	= 0;
	static uint8_t T100ms_counter	= 0;
	
	// 40us
	T1ms_counter++;
	
	if(T1ms_counter == 25) {
		T1ms_counter = 0;
		T1ms_flag = 1;
		
		// 1ms
		T10ms_counter++;
		
		// 10ms
	
		if(T10ms_counter == 10) {
			T10ms_counter = 0;
			T10ms_flag = 1;	
			
			T100ms_counter++;
		}
		
		// 100ms
		
		if(T100ms_counter == 10) {
			T100ms_counter = 0;
			T100ms_flag = 1;
		}
		
	}
}
